/** @param {jQuery} $ jQuery Object */
!function($, window, document, _undefined)
{
	/**
	 * Invoice item listener for the options page. This handles automatically
	 * creating additional text boxes when necessary.
	 *
	 * @param jQuery tr.InvoiceItemListener to listen to
	 */
	XenForo.InvoiceItemListener = function($element) {
		this.__construct($element);
	};
	XenForo.InvoiceItemListener.prototype =
	{
		__construct: function($element)
		{
			if (!$element.find('input.InvoiceItemId').length)
			{
				$element.one('keypress', $.context(this, 'createChoice'));
			}
			
			$element.find('input.InvoiceItemTotal').change(function(e)
			{
				$costTextbox = $element.find('input.InvoiceItemCost').first();
				if (!isNaN(parseFloat($costTextbox.val()))) {
					$costTextbox.val(parseFloat($costTextbox.val()).toFixed(2));
				} else {
					$costTextbox.val('0.00');
				}
				$qtyTextbox = $element.find('input.InvoiceItemQuantity').first();
				if (!isNaN(parseFloat($costTextbox.val())) && parseFloat($qtyTextbox.val()) > 0) {
					$qtyTextbox.val(Math.round(parseFloat($qtyTextbox.val())*1000)/1000);
				} else {
					$qtyTextbox.val('1');
				}
				$total = $element.find('td.InvoiceItemTotal').first();
				if ($costTextbox.val() && $qtyTextbox.val())
				{
					var amount = parseFloat($costTextbox.val()) * parseFloat($qtyTextbox.val());
					$total.html(parseFloat(amount).toFixed(2));
				}
			});

			this.$element = $element;
			if (!this.$base)
			{
				this.$base = $element.clone();
			}
		},
		
		createChoice: function()
		{
			var $new = this.$base.clone(),
				nextCounter = this.$element.parent().children().length;
			$new.find('input[name]').each(function()
			{
				var $this = $(this);
				$this.attr('name', $this.attr('name').replace(/\[(\d+)\]/, '[' + nextCounter + ']'));
			});
			$new.find('textarea[name]').each(function()
			{
				var $this = $(this);
				$this.attr('name', $this.attr('name').replace(/\[(\d+)\]/, '[' + nextCounter + ']'));
			});

			$new.find('*[id]').each(function()
			{
				var $this = $(this);
				$this.removeAttr('id');
				XenForo.uniqueId($this);

				if (XenForo.formCtrl)
				{
					XenForo.formCtrl.clean($this);
				}
			});

			$new.xfInsert('insertAfter', this.$element);

			this.__construct($new);
		}
	};

	/**
	 * Invoice item field listener for the options page. This handles automatically
	 * creating additional text boxes when necessary.
	 *
	 * @param jQuery div.InvoiceItemFieldListener to listen to
	 */
	XenForo.InvoiceItemFieldListener = function($element)
	{
		this.__construct($element);		
	};
	
	XenForo.InvoiceItemFieldListener.prototype =
	{
		__construct: function($element)
		{
			$element.one('keypress', $.context(this, 'createChoice'));

			this.$element = $element;
			if (!this.$base)
			{
				this.$base = $element.clone();
			}
		},

		createChoice: function()
		{
			var $new = this.$base.clone(),
				nextCounter = this.$element.parent().children().length;
			$new.find('input[name]').each(function()
			{
				var $this = $(this);
				$this.attr('name', $this.attr('name').replace(/\[(\d+)\]\[(\d+)\]/, '[$1][' + nextCounter + ']'));
			});
			
			
			$new.find('select[name]').each(function()
			{
				var $this = $(this);
				$this.attr('name', $this.attr('name').replace(/\[(\d+)\]\[(\d+)\]/, '[$1][' + nextCounter + ']'));
			});

			$new.find('*[id]').each(function()
			{
				var $this = $(this);
				$this.removeAttr('id');
				XenForo.uniqueId($this);

				if (XenForo.formCtrl)
				{
					XenForo.formCtrl.clean($this);
				}
			});

			$new.xfInsert('insertAfter', this.$element);

			this.__construct($new);
		}
	};	


	// *********************************************************************

	XenForo.register('tr.InvoiceItemDataRow', 'XenForo.InvoiceItemDataRow');
	XenForo.register('tr.InvoiceItemListener', 'XenForo.InvoiceItemListener');
	XenForo.register('div.InvoiceItemFieldListener', 'XenForo.InvoiceItemFieldListener');

}
(jQuery, this, document);