<?php

/**
 * Controller for handling the recurring invoice section in the admin control
 * panel.
 */
class Waindigo_Invoices_ControllerAdmin_Recurring_Invoice extends Waindigo_Invoices_ControllerAdmin_Invoice
{

    protected $_invoiceDataWriter = 'Waindigo_Invoices_DataWriter_Recurring_Invoice';

    protected $_invoiceItemDataWriter = 'Waindigo_Invoices_DataWriter_Recurring_Invoice_Item';

    protected $_invoiceItemFieldDataWriter = 'Waindigo_Invoices_DataWriter_Recurring_Invoice_Item_Field';

    public function actionIndex()
    {
        return $this->responseRedirect(XenForo_ControllerResponse_Redirect::RESOURCE_CANONICAL_PERMANENT,
            XenForo_Link::buildAdminLink('invoices'));
    } /* END actionIndex */

    public function actionViewStandingOrder()
    {
        $input = $this->_input->filter(array(
            'invoice_id' => XenForo_Input::UINT
        ));

        $invoice = $this->_getInvoiceOrError($input['invoice_id']);

        $invoiceModel = $this->_getInvoiceModel();
        $invoice = $invoiceModel->prepareInvoice($invoice);

        $this->_routeMatch->setResponseType('raw');

        $viewParams = array(
            'pdf' => $this->_getInvoiceModel()->getStandingOrderPdf($invoice)
        );

        return $this->responseView('Waindigo_Invoices_ViewAdmin_Recurring_Invoice_ViewStandingOrder', '', $viewParams);
    } /* END actionViewStandingOrder */

    protected function _getNoInvoicesError()
    {
        return $this->responseError(new XenForo_Phrase('waindigo_there_are_no_recurring_invoices_invoices'));
    } /* END _getNoInvoicesError */

    protected function _getInvoiceIndexResponse($viewParams = array())
    {
        return $this->responseView('Waindigo_Invoices_ViewAdmin_Recurring_Invoice_List',
            'waindigo_recurring_invoice_list_invoices', $viewParams);
    } /* END _getInvoiceIndexResponse */

    /**
     *
     * @see Waindigo_Invoices_ControllerAdmin_Invoice::_getInvoiceChoiceResponse()
     */
    protected function _getInvoiceChoiceResponse(array $choice = array())
    {
        /* @var $responseView XenForo_ControllerResponse_View */
        $responseView = parent::_getInvoiceChoiceResponse($choice);

        $responseView->viewName = 'Waindigo_Invoices_ViewAdmin_Recurring_Invoice_Choice';
        $responseView->templateName = 'waindigo_recurring_invoice_choice_invoices';

        return $responseView;
    } /* END _getInvoiceChoiceResponse */ /* END _getInvoiceAddEditResponse */


    protected function _getInvoiceChoiceRedirect(array $choice = array())
    {
        return $this->responseRedirect(XenForo_ControllerResponse_Redirect::RESOURCE_CANONICAL_PERMANENT,
            XenForo_Link::buildAdminLink('recurring-invoices/add', '', $choice));
    } /* END _getInvoiceChoiceRedirect */

    /**
     *
     * @see Waindigo_Invoices_ControllerAdmin_Invoice::_getInvoiceAddEditResponse()
     */
    protected function _getInvoiceAddEditResponse(array $invoice = array())
    {
        /* @var $responseView XenForo_ControllerResponse_View */
        $responseView = parent::_getInvoiceAddEditResponse($invoice);

        $responseView->viewName = 'Waindigo_Invoices_ViewAdmin_Recurring_Invoice_Edit';
        $responseView->templateName = 'waindigo_recurring_invoice_edit_invoices';

        return $responseView;
    } /* END _getInvoiceAddEditResponse */

    protected function _getInvoiceData()
    {
        $inputFields = $this->_getDefaultInvoiceInputFields();

        $inputFields['active'] = XenForo_Input::UINT;
        $inputFields['length_amount'] = XenForo_Input::UINT;
        $inputFields['length_unit'] = XenForo_Input::STRING;

        return $this->_input->filter($inputFields);
    } /* END _getInvoiceData */

    protected function _getInvoiceIndexRedirect($invoiceId = null)
    {
        return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS,
            XenForo_Link::buildAdminLink('recurring-invoices/list') . ($invoiceId ? $this->getLastHash($invoiceId) : ''));
    } /* END _getInvoiceIndexRedirect */

    /**
     * Deletes the specified invoice.
     *
     * @return XenForo_ControllerResponse_Abstract
     */
    public function actionDelete()
    {
        $invoiceId = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);
        $invoice = $this->_getInvoiceOrError($invoiceId);

        $writer = XenForo_DataWriter::create($this->_invoiceDataWriter, XenForo_DataWriter::ERROR_EXCEPTION);
        $writer->setExistingData($invoice);

        if ($this->isConfirmedPost()) {
            return $this->_deleteData($this->_invoiceDataWriter, 'invoice_id',
                XenForo_Link::buildAdminLink('recurring-invoices/list'));
        } else {
            $viewParams = array(
                'invoice' => $invoice
            );

            return $this->responseView('Waindigo_Invoices_ViewAdmin_Recurring_Invoice_Delete',
                'waindigo_recurring_invoice_delete_invoices', $viewParams);
        }
    } /* END actionDelete */

    /**
     * Gets the specified customer or throws an exception.
     *
     * @param integer $invoiceId
     *
     * @return array
     */
    protected function _getInvoiceOrError($invoiceId)
    {
        $info = $this->_getInvoiceModel()->getInvoiceById($invoiceId);
        if (!$info) {
            throw $this->responseException(
                $this->responseError(new XenForo_Phrase('waindigo_requested_recurring_invoice_not_found_invoices'), 404));
        }

        return $info;
    } /* END _getInvoiceOrError */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice
     */
    protected function _getInvoiceModel()
    {
        return $this->_getRecurringInvoiceModel();
    } /* END _getInvoiceModel */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice_Item_Field
     */
    protected function _getInvoiceItemFieldModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice_Item_Field');
    } /* END _getInvoiceItemFieldModel */
}