<?php

class Waindigo_Invoices_DataWriter_Invoice extends XenForo_DataWriter
{

    protected $_invoiceItemDataWriter = 'Waindigo_Invoices_DataWriter_Invoice_Item';

    protected $_invoiceItemWriters = array();

    protected function _getFields()
    {
        return array(
            'invoice_waindigo' => array(
                'invoice_id' => array(
                    'type' => self::TYPE_UINT,
                    'autoIncrement' => true
                ),
                'invoice_number' => array(
                    'type' => self::TYPE_UINT,
                    'autoIncrement' => true
                ),
                'user_id' => array(
                    'type' => self::TYPE_UINT,
                    'required' => true
                ),
                'username' => array(
                    'type' => self::TYPE_STRING,
                    'required' => true
                ),
                'name' => array(
                    'type' => self::TYPE_STRING,
                    'required' => true
                ),
                'address' => array(
                    'type' => self::TYPE_STRING,
                    'default' => ''
                ),
                'to_user_id' => array(
                    'type' => self::TYPE_UINT,
                    'required' => true
                ),
                'to_username' => array(
                    'type' => self::TYPE_STRING,
                    'required' => true
                ),
                'to_name' => array(
                    'type' => self::TYPE_STRING,
                    'required' => true
                ),
                'to_address' => array(
                    'type' => self::TYPE_STRING,
                    'default' => ''
                ),
                'amount' => array(
                    'type' => self::TYPE_UINT,
                    'default' => 0
                ),
                'invoice_date' => array(
                    'type' => self::TYPE_STRING,
                    'required' => true
                ),
                'paid_state' => array(
                    'type' => self::TYPE_STRING,
                    'default' => 'unpaid',
                    'allowedValues' => array(
                        'paid',
                        'uncleared',
                        'unpaid'
                    )
                ),
                'paid_date' => array(
                    'type' => self::TYPE_UINT,
                    'default' => 0
                ),
                'invoice_state' => array(
                    'type' => self::TYPE_STRING,
                    'default' => 'visible',
                    'allowedValues' => array(
                        'visible',
                        'deleted',
                        'draft'
                    )
                )
            )
        );
    } /* END _getFields */

    protected function _getExistingData($data)
    {
        if (!$id = $this->_getExistingPrimaryKey($data)) {
            return false;
        }

        /* @var $customerModel Waindigo_Invoices_Model_Invoice */
        $invoiceModel = $this->getModelFromCache('Waindigo_Invoices_Model_Invoice');

        return array(
            'invoice_waindigo' => $invoiceModel->getInvoiceById($id)
        );
    } /* END _getExistingData */

    protected function _getUpdateCondition($tableName)
    {
        return 'invoice_id = ' . $this->_db->quote($this->getExisting('invoice_id'));
    } /* END _getUpdateCondition */

    /**
     * Updates the customer invoice count.
     */
    protected function _updateCustomerInvoiceCount()
    {
        /*
         * $this->_db->query(' UPDATE and_customer SET invoice_count =
         * invoice_count + 1 WHERE customer_id = ? ',
         * $this->get('from_customer_id')); $customerId =
         * $this->get('from_customer_id'); $this->_db->query(' UPDATE
         * and_customer SET invoice_count = invoice_count + 1 WHERE customer_id
         * = ? ', $this->get('to_customer_id'));
         */
    } /* END _updateCustomerInvoiceCount */

    protected function _updateNextInvoiceNumber()
    {
        /*
         * if ($this->isUpdate()) { $this->_db->query(' UPDATE and_customer SET
         * next_invoice_number = GREATEST(next_invoice_number, ?) WHERE
         * customer_id = ? ', array($this->get('invoice_number')+1,
         * $this->get('from_customer_id'))); } else { $this->_db->query(' UPDATE
         * and_customer SET next_invoice_number =
         * GREATEST(next_invoice_number+1, ?) WHERE customer_id = ? ',
         * array($this->get('invoice_number')+1,
         * $this->get('from_customer_id'))); }
         */
    } /* END _updateNextInvoiceNumber */

    protected function _preSave()
    {
        if ($this->isChanged('invoice_number') && $this->get('invoice_number')) {
            $invoiceNumber = $this->get('invoice_number');
            $userId = $this->get('user_id');
            $conditions = array(
                'invoice_number' => $invoiceNumber,
                'user_id' => $userId
            );
            $invoice = $this->_getInvoiceModel()->getInvoice($conditions);
            if (!empty($invoice) && $invoice['invoice_id'] != $this->get('invoice_id')) {
                $this->error('Invoice number already in use.', 'invoice_number');
            }
        } else
            if (!$this->get('invoice_number')) {
                $this->set('invoice_number', $this->_getNextInvoiceNumber());
            }

        // if (empty($this->_invoiceItemWriters)) {
        // $this->error(new
    // XenForo_Phrase('waindigo_must_have_at_least_one_item_invoices'));
        // }
    } /* END _preSave */

    protected function _postSave()
    {
        if (!$this->isUpdate()) {
            $this->_updateCustomerInvoiceCount();
        }
        $this->_updateNextInvoiceNumber();

        $this->_alertUser();

        foreach ($this->_invoiceItemWriters as $itemWriter) {
            $itemWriter->set('invoice_id', $this->get('invoice_id'));
            $itemWriter->save();
        }

        parent::_postSave();
    } /* END _postSave */

    protected function _alertUser()
    {
        /* @var $alertModel XenForo_Model_Alert */
        /*$alertModel = XenForo_Model::create('XenForo_Model_Alert');
        $alerts = $alertModel->getAlertsForUser($this->get('to_user_id'),
            XenForo_Model_Alert::FETCH_MODE_ALL);
        foreach ($alerts as $alert) {
            if (isset($alert['content']['invoice_id']) && $alert['content']['invoice_id'] == $this->get('invoice_id')) {
                return;
            }
        }*/

        if ($this->isInsert()) {
            XenForo_Model_Alert::alert($this->get('to_user_id'),
                $this->get('user_id'),
                $this->get('username'), 'invoice', $this->get('invoice_id'), 'new');
        }

        /*$params = array(
            'sender' => $this->get('name'),
            'receiver' => $this->get('to_name')
        );*/
        //$mail = XenForo_Mail::create('invoice_waindigo_create', $params,
        //    $customers[$this->get('to_customer_id')]['language_id']);
        //$mail->send($customers[$this->get('to_customer_id')]['email'],
        //    $customers[$this->get('to_customer_id')]['username']);
    } /* END _alertUser */

    public function setInvoiceItems(array $invoiceItems, array $invoiceItemFields)
    {
        $totalCostAmount = 0;
        foreach ($invoiceItems as $invoiceItemRow => $invoiceItem) {
            if (isset($invoiceItem['invoice_item_id']) || $invoiceItem['quantity']) {
                $itemWriter = XenForo_DataWriter::create($this->_invoiceItemDataWriter);

                if (isset($invoiceItem['invoice_item_id'])) {
                    $itemWriter->setExistingData($invoiceItem['invoice_item_id']);
                }

                $costAmount = (float) $invoiceItem['cost_amount'];
                if (isset($invoiceItem['invoice_item_id']) && !$costAmount) {
                    $itemWriter->delete();
                } else
                    if ($costAmount) {
                        $itemWriter->bulkSet($invoiceItem);
                        $itemWriter->setInvoiceItemFields($invoiceItemFields[$invoiceItemRow]);
                        $totalCostAmount += $costAmount * $invoiceItem['quantity'];
                        $this->_invoiceItemWriters[] = $itemWriter;
                    }
            }
        }
        $this->set('amount', $totalCostAmount);
    } /* END setInvoiceItems */

    protected function _getNextInvoiceNumber()
    {
        return $this->_getInvoiceModel()->getNextInvoiceNumberForUserId($this->get('user_id'));
    } /* END _getNextInvoiceNumber */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice
     */
    protected function _getInvoiceModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice');
    } /* END _getInvoiceModel */
}