<?php

class Waindigo_Invoices_DataWriter_Invoice_Item extends XenForo_DataWriter
{

    protected $_invoiceItemFieldDataWriter = 'Waindigo_Invoices_DataWriter_Invoice_Item_Field';

    protected $_invoiceItemFieldWriters = array();

    protected function _getFields()
    {
        return array(
            'invoice_item_waindigo' => array(
                'invoice_item_id' => array(
                    'type' => self::TYPE_UINT,
                    'autoIncrement' => true
                ),
                'invoice_id' => array(
                    'type' => self::TYPE_UINT,
                    'required' => true
                ),
                'description' => array(
                    'type' => self::TYPE_STRING,
                    'default' => ''
                ),
                'cost_amount' => array(
                    'type' => self::TYPE_FLOAT,
                    'default' => '0.00'
                ),
                'quantity' => array(
                    'type' => self::TYPE_FLOAT,
                    'default' => '0.000'
                )
            )
        );
    } /* END _getFields */

    protected function _getExistingData($data)
    {
        if (!$id = $this->_getExistingPrimaryKey($data)) {
            return false;
        }

        /* @var $customerModel Waindigo_Invoices_Model_Invoice_Item */
        $invoiceItemModel = $this->getModelFromCache('Waindigo_Invoices_Model_Invoice_Item');

        return array(
            'invoice_item_waindigo' => $invoiceItemModel->getInvoiceItemById($id)
        );
    } /* END _getExistingData */

    protected function _getUpdateCondition($tableName)
    {
        return 'invoice_item_id = ' . $this->_db->quote($this->getExisting('invoice_item_id'));
    } /* END _getUpdateCondition */

    public function setInvoiceItemFields(array $invoiceItemFields)
    {
        $fieldNames = array();
        foreach ($invoiceItemFields as $invoiceItemField) {
            if ($invoiceItemField['field_name']) {
                $fieldName = XenForo_Link::getTitleForUrl($invoiceItemField['field_name'], true);
                if (!in_array($fieldName, $fieldNames)) {
                    $fieldNames[] = $fieldName;
                } else {
                    $this->error(
                        new XenForo_Phrase('waindigo_invoice_item_field_names_must_be_unique_x_invoices',
                            array(
                                'field' => $fieldName
                            )), $fieldName);
                }
            }
        }
        foreach ($invoiceItemFields as $invoiceItemField) {
            if (isset($invoiceItemField['invoice_item_field_id']) || $invoiceItemField['field_name']) {
                $fieldWriter = XenForo_DataWriter::create($this->_invoiceItemFieldDataWriter);

                if (isset($invoiceItemField['invoice_item_field_id'])) {
                    $fieldWriter->setExistingData($invoiceItemField['invoice_item_field_id']);
                } elseif ($invoiceItemField['field_name']) {
                    $existingInvoiceItemField = $this->_getInvoiceItemFieldModel()->getInvoiceItemField(array(
                    	'invoice_item_id' => $this->get('invoice_item_id'),
                        'field_name' => $invoiceItemField['field_name']
                    ));
                    if ($existingInvoiceItemField) {
                        $fieldWriter->setExistingData($existingInvoiceItemField);
                    }
                }

                if (!$invoiceItemField['field_name']) {
                    if ($fieldWriter->isUpdate()) {
                        $fieldWriter->delete();
                    }
                } else {
                    $invoiceItemField = Waindigo_Invoices_Model_Invoice_Item_Field::processFieldValue($invoiceItemField);
                    $fieldWriter->bulkSet($invoiceItemField);
                    $this->_invoiceItemFieldWriters[] = $fieldWriter;
                }
            }
        }
    } /* END setInvoiceItemFields */

    protected function _postSave()
    {
        foreach ($this->_invoiceItemFieldWriters as $fieldWriter) {
            $fieldWriter->set('invoice_item_id', $this->get('invoice_item_id'));
            $fieldWriter->save();
        }
        parent::_postSave();
    } /* END _postSave */

    protected function _preDelete()
    {
        $invoiceItemFields = $this->_getInvoiceItemFieldModel()->getInvoiceItemFields(
            array(
                'invoice_item_id' => $this->getExisting('invoice_item_id')
            ));

        foreach ($invoiceItemFields as $invoiceItemField) {
            $dw = XenForo_DataWriter::create('Waindigo_Invoices_DataWriter_Invoice_Item_Field');
            $dw->setExistingData($invoiceItemField);
            $dw->delete();
        }

        $invoiceItems = $this->_getInvoiceItemModel()->getInvoiceItems(
            array(
                'invoice_id' => $this->getExisting('invoice_id')
            ));

        unset($invoiceItems[$this->getExisting('invoice_item_id')]);

        if (empty($invoiceItems)) {
            $dw = XenForo_DataWriter::create('Waindigo_Invoices_DataWriter_Invoice');
            $dw->setExistingData($this->getExisting('invoice_id'));
            $dw->delete();
        }
    } /* END _preDelete */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice_Item_Field
     */
    protected function _getInvoiceItemFieldModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice_Item_Field');
    } /* END _getInvoiceItemFieldModel */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice_Item
     */
    protected function _getInvoiceItemModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice_Item');
    } /* END _getInvoiceItemModel */
}