<?php

class Waindigo_Invoices_DataWriter_Recurring_Invoice_Item extends Waindigo_Invoices_DataWriter_Invoice_Item
{

    protected $_invoiceItemFieldDataWriter = 'Waindigo_Invoices_DataWriter_Recurring_Invoice_Item_Field';

    protected function _getFields()
    {
        return array(
            'recurring_invoice_item_waindigo' => array(
                'invoice_item_id' => array(
                    'type' => self::TYPE_UINT,
                    'autoIncrement' => true
                ),
                'invoice_id' => array(
                    'type' => self::TYPE_UINT,
                    'required' => true
                ),
                'description' => array(
                    'type' => self::TYPE_STRING,
                    'default' => ''
                ),
                'cost_amount' => array(
                    'type' => self::TYPE_FLOAT,
                    'default' => '0.00'
                ),
                'quantity' => array(
                    'type' => self::TYPE_FLOAT,
                    'default' => '0.000'
                )
            )
        );
    } /* END _getFields */

    protected function _getExistingData($data)
    {
        if (!$id = $this->_getExistingPrimaryKey($data)) {
            return false;
        }

        /*
         * @var $invoiceItemModel Waindigo_Invoices_Model_Recurring_Invoice_Item
         */
        $invoiceItemModel = $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice_Item');

        return array(
            'recurring_invoice_item_waindigo' => $invoiceItemModel->getInvoiceItemById($id)
        );
    } /* END _getExistingData */

    protected function _getUpdateCondition($tableName)
    {
        return 'invoice_item_id = ' . $this->_db->quote($this->getExisting('invoice_item_id'));
    } /* END _getUpdateCondition */

    protected function _preDelete()
    {
        $invoiceItemFields = $this->_getInvoiceItemFieldModel()->getInvoiceItemFields(
            array(
                'invoice_item_id' => $this->getExisting('invoice_item_id')
            ));

        foreach ($invoiceItemFields as $invoiceItemField) {
            $dw = XenForo_DataWriter::create('Waindigo_Invoices_DataWriter_Recurring_Invoice_Item_Field');
            $dw->setExistingData($invoiceItemField['invoice_item_field_id']);
            $dw->delete();
        }

        $invoiceItems = $this->_getInvoiceItemModel()->getInvoiceItems(
            array(
                'invoice_id' => $this->getExisting('invoice_id')
            ));

        unset($invoiceItems[$this->getExisting('invoice_item_id')]);

        if (empty($invoiceItems)) {
            $dw = XenForo_DataWriter::create('Waindigo_Invoices_DataWriter_Recurring_Invoice');
            $dw->setExistingData($this->getExisting('invoice_id'));
            $dw->delete();
        }
    } /* END _preDelete */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice_Item_Field
     */
    protected function _getInvoiceItemFieldModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice_Item_Field');
    } /* END _getInvoiceItemFieldModel */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice_Item
     */
    protected function _getInvoiceItemModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice_Item');
    } /* END _getInvoiceItemModel */
}