<?php

class Waindigo_Invoices_Extend_XenForo_ControllerPublic_Account extends XFCP_Waindigo_Invoices_Extend_XenForo_ControllerPublic_Account
{

    /**
     * Displays a list of visitor's invoices.
     *
     * @return XenForo_ControllerResponse_View
     */
    public function actionInvoices()
    {
        $invoiceId = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);
        if ($invoiceId) {
            return $this->responseReroute(__CLASS__, 'invoice-view');
        }

        $visitor = XenForo_Visitor::getInstance();
        $invoiceModel = $this->_getInvoiceModel();

        $conditions = $this->_getInvoicesListConditions();
        $fetchOptions = $this->_getInvoicesListFetchOptions();

        $totalInvoices = $invoiceModel->countInvoices($conditions);

        $invoices = $invoiceModel->getInvoices($conditions, $fetchOptions);
        $invoices = $invoiceModel->prepareInvoices($invoices);

        $viewParams = array(
            'invoices' => $invoices,
            'page' => $fetchOptions['page'],
            'invoicesPerPage' => $fetchOptions['perPage'],
            'totalInvoices' => $totalInvoices
        );

        return $this->_getWrapper('account', 'invoices',
            $this->responseView('Waindigo_Invoices_ViewPublic_Invoice_List', 'waindigo_invoice_list_invoices',
                $viewParams));
    } /* END actionInvoices */

    public function actionInvoiceView()
    {
        $invoiceId = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);

        $invoiceModel = $this->_getInvoiceModel();
        $invoice = $invoiceModel->getInvoiceById($invoiceId);

        if (!$invoice) {
            return $this->responseError(new XenForo_Phrase('waindigo_requested_invoice_not_found_invoices'), 404);
        }

        if (!$invoiceModel->canViewInvoice($invoice)) {
            return $this->responseNoPermission();
        }

        $invoice = $invoiceModel->prepareInvoice($invoice);

        $this->_routeMatch->setResponseType('raw');

        $viewParams = array(
            'pdf' => $this->_getInvoiceModel()->getInvoicePdf($invoice)
        );

        return $this->responseView('Waindigo_Invoices_ViewPublic_Account_Invoice_View', '', $viewParams);
    } /* END actionInvoiceView */

    /**
     * Displays a list of visitor's recurring invoices.
     *
     * @return XenForo_ControllerResponse_View
     */
    public function actionRecurringInvoices()
    {
        $invoiceId = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);
        if ($invoiceId) {
            return $this->responseReroute(__CLASS__, 'invoice-view');
        }

        $visitor = XenForo_Visitor::getInstance();
        $invoiceModel = $this->_getRecurringInvoiceModel();

        $conditions = $this->_getInvoicesListConditions();
        $fetchOptions = $this->_getInvoicesListFetchOptions();
        $conditions['active'] = true;

        $totalInvoices = $invoiceModel->countInvoices($conditions);

        $invoices = $invoiceModel->getInvoices($conditions, $fetchOptions);
        $invoices = $invoiceModel->prepareInvoices($invoices);

        $viewParams = array(
            'invoices' => $invoices,
            'page' => $fetchOptions['page'],
            'invoicesPerPage' => $fetchOptions['perPage'],
            'totalInvoices' => $totalInvoices
        );

        return $this->_getWrapper('account', 'recurring-invoices',
            $this->responseView('Waindigo_Invoices_ViewPublic_Invoice_List', 'waindigo_recurring_invoice_list_invoices',
                $viewParams));
    } /* END actionRecurringInvoices */

    public function actionRecurringInvoiceView()
    {
        $invoiceId = $this->_input->filterSingle('invoice_id', XenForo_Input::UINT);

        $invoiceModel = $this->_getRecurringInvoiceModel();
        $invoice = $invoiceModel->getInvoiceById($invoiceId);

        if (!$invoice) {
            return $this->responseError(new XenForo_Phrase('waindigo_invoicing_requested_invoice_not_found_invoices'), 404);
        }

        if (!$invoiceModel->canViewInvoice($invoice)) {
            return $this->responseNoPermission();
        }

        $invoice = $invoiceModel->prepareInvoice($invoice);

        $this->_routeMatch->setResponseType('raw');

        $viewParams = array(
            'pdf' => $invoiceModel->getStandingOrderPdf($invoice)
        );

        return $this->responseView('Waindigo_Invoices_ViewPublic_Account_RecurringInvoice_View', '', $viewParams);
    } /* END actionRecurringInvoiceView */

    protected function _getInvoicesListConditions()
    {
        $conditions['any_user_id'] = XenForo_Visitor::getUserId();

        return $conditions;
    } /* END _getInvoicesListConditions */

    protected function _getInvoicesListFetchOptions()
    {
        return array(
            'page' => $this->_input->filterSingle('page', XenForo_Input::UINT),
            'perPage' => 20,
            'order' => 'invoice_id'
        );
    } /* END _getInvoicesListFetchOptions */

    /**
     *
     * @return Waindigo_Invoices_Model_Invoice
     */
    protected function _getInvoiceModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Invoice');
    } /* END _getInvoiceModel */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice
     */
    protected function _getRecurringInvoiceModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice');
    } /* END _getRecurringInvoiceModel */
}