<?php

class Waindigo_Invoices_Extend_XenForo_Model_UserUpgrade extends XFCP_Waindigo_Invoices_Extend_XenForo_Model_UserUpgrade
{

    /**
     *
     * @see XenForo_Model_UserUpgrade::upgradeUser()
     */
    public function upgradeUser($userId, array $upgrade, $allowInsertUnpurchasable = false, $endDate = null)
    {
        $upgradeRecordId = parent::upgradeUser($userId, $upgrade, $allowInsertUnpurchasable, $endDate);
        $upgradeRecord = $this->getActiveUserUpgradeRecordById($upgradeRecordId);

        $this->_createUserUpgradeInvoice($userId, $upgrade, $upgradeRecord);

        return $upgradeRecordId;
    } /* END upgradeUser */

    /**
     * Creates an invoice for the specified upgrade.
     *
     * @param integer $userId
     * @param array $upgrade Info about upgrade to apply
     * @param array $upgradeRecord User upgrade record
     *
     * @return integer false ID
     */
    protected function _createUserUpgradeInvoice($userId, array $upgrade, array $upgradeRecord)
    {
        $db = $this->_getDb();

        $_request = new Zend_Controller_Request_Http();
        $_input = new XenForo_Input($_request);

        $name = $_input->filter(
            array(
                'first_name' => XenForo_Input::STRING,
                'last_name' => XenForo_Input::STRING
            ));

        $address = $_input->filter(
            array(
                'address_street' => XenForo_Input::STRING,
                'address_city' => XenForo_Input::STRING,
                'address_state' => XenForo_Input::STRING,
                'address_zip' => XenForo_Input::STRING,
                'address_country' => XenForo_Input::STRING
            ));

        $data = array(
            'user_id' => $userId,
            'name' => 'Test',
            'address' => 'Test',
            'to_user_id' => $userId,
            'to_name' => implode(' ', array_filter($name)),
            'to_address' => implode("\n", array_filter($address)),
            'invoice_date' => date('Y-m-d', XenForo_Application::$time)
        // 'currency_id' => $upgrade['cost_currency'],
                );

        /* @var $userModel XenForo_Model_User */
        $userModel = XenForo_Model::create('XenForo_Model_User');

        $user = $userModel->getUserById($data['user_id']);
        $toUser = $userModel->getUserById($data['to_user_id']);

        $data['username'] = $user['username'];
        $data['to_username'] = $toUser['username'];
        if (!$data['to_name']) {
            $data['to_name'] = $data['to_username'];
        }

        $writer = XenForo_DataWriter::create('Waindigo_Invoices_DataWriter_Invoice');
        $writer->bulkSet($data);

        $invoiceItems = array(
            0 => array(
                'description' => new XenForo_Phrase('account_upgrade'),
                'cost_amount' => $upgrade['cost_amount'],
                'quantity' => 1
            )
        );

        $invoiceItemFields = array(
            0 => array(
                array(
                    'field_name' => 'user_upgrade_record_id',
                    'field_value' => $upgradeRecord['user_upgrade_record_id']
                ),
                array(
                    'field_name' => 'end_date',
                    'field_value' => $upgradeRecord['end_date']
                )
            )
        );

        $writer->setInvoiceItems($invoiceItems, $invoiceItemFields);
        $writer->save();

        return $db->lastInsertId();
    } /* END _createUserUpgradeInvoice */
}