<?php

/**
 * Model for recurring invoices.
 */
class Waindigo_Invoices_Model_Recurring_Invoice extends Waindigo_Invoices_Model_Invoice
{

    /**
     *
     * @var string Recurring invoice table name
     */
    protected $_tableName = 'recurring_invoice_waindigo';

    public function getStandingOrderPdf(array $invoice)
    {
        /* @var $pdf Waindigo_Invoices_Pdf_Invoice */
        $pdf = Waindigo_Pdf::create('Waindigo_Invoices_Pdf_StandingOrder');

        $pdf->draw($invoice);

        return $pdf;
    } /* END getStandingOrderPdf */

    /**
     * Prepares join-related fetch options.
     *
     * @param array $fetchOptions
     *
     * @return array Containing 'selectFields' and 'joinTables' keys.
     */
    public function prepareInvoiceFetchOptions(array $fetchOptions)
    {
        $sharedFetchOptions = $this->prepareInvoiceSharedFetchOptions($fetchOptions);
        $selectFields = $sharedFetchOptions['selectFields'];
        $joinTables = $sharedFetchOptions['joinTables'];
        $orderBy = $sharedFetchOptions['orderBy'];

        if (!empty($fetchOptions['join'])) {
            if ($fetchOptions['join'] & self::FETCH_INVOICE_ITEMS) {
                $selectFields .= ',
				invoice_item.*';
                $joinTables .= '
				LEFT JOIN recurring_invoice_item_waindigo AS invoice_item ON (invoice.invoice_id = invoice_item.invoice_id)';
            }
            if ($fetchOptions['join'] & self::FETCH_INVOICE_ITEMS_AND_FIELDS) {
                $selectFields .= ',
				invoice_item_field.*';
                $joinTables .= '
				LEFT JOIN recurring_invoice_item_field_waindigo AS invoice_item_field ON (invoice_item.invoice_item_id = invoice_item_field.invoice_item_id)';
            }
        }

        return array(
            'selectFields' => $selectFields,
            'joinTables' => $joinTables,
            'orderClause' => ($orderBy ? "ORDER BY $orderBy" : '')
        );
    } /* END prepareInvoiceFetchOptions */

    /**
     * Prepares a set of conditions to select users against.
     *
     * @param array $conditions List of conditions.
     * @param array $fetchOptions The fetch options that have been provided. May
     * be edited if criteria requires.
     *
     * @return string Criteria as SQL for where clause
     */
    public function prepareInvoiceConditions(array $conditions, array &$fetchOptions)
    {
        $db = $this->_getDb();
        $sqlConditions = $this->prepareInvoiceSharedConditions($conditions, $fetchOptions);

        if (isset($conditions['active'])) {
            $sqlConditions[] = 'invoice.active = 1';
        }

        if (isset($conditions['length_amount'])) {
            $sqlConditions[] = 'invoice.length_amount = ' . $db->quote($conditions['length_amount']);
        }

        if (isset($conditions['length_unit'])) {
            $sqlConditions[] = 'invoice.length_unit = ' . $db->quote($conditions['length_unit']);
        }

        return $this->getConditionsForClause($sqlConditions);
    } /* END prepareInvoiceConditions */

    /**
     *
     * @return Waindigo_Invoices_Model_Recurring_Invoice_Item
     */
    protected function _getInvoiceItemModel()
    {
        return $this->getModelFromCache('Waindigo_Invoices_Model_Recurring_Invoice_Item');
    } /* END _getInvoiceItemModel */
}