<?php

class Waindigo_Invoices_Pdf_Block_Invoice_Header extends Waindigo_Pdf_Block
{

    public function draw(Waindigo_Pdf $pdf, array $invoice = array(), $pageNumber = 1)
    {
        parent::draw($pdf);

        if ($invoice['invoice_state'] != 'visible') {
            if ($invoice['invoice_state'] == 'draft') {
                $string = 'DRAFT';
            } else {
                $string = 'VOID';
            }
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
            $fontSize = 200;
            $width = self::widthForStringUsingFontSize($string, $font, $fontSize);
            $height = self::getFontSizeHeight($font, $fontSize);
            $x = $this->_pdf->getPageWidth(null, false) / 2;
            $y = $this->_pdf->getPageHeight(null, false) / 2;
            $this->_page->rotate($x, $y, 1);
            $this->_page->setFont($font, $fontSize)->drawText($string, $x - $width / 2, $y - $height / 2, 'UTF-8');
            $this->_page->rotate($x, $y, -1);
        }

        // logo
        if (isset($invoice['user']['user_id'])) {
            /* @var $avatarModel XenForo_Model_Avatar */
            $avatarModel = XenForo_Model::create('XenForo_Model_Avatar');
            $imageFileName = $avatarModel->getAvatarFilePath($invoice['user']['user_id'], 'l');
            if (file_exists($imageFileName)) {
                $size = getimagesize($imageFileName);
                $image = false;
                if ($size['mime'] == "image/jpeg") {
                    $image = new Zend_Pdf_Resource_Image_Jpeg($imageFileName);
                } else
                    if ($size['mime'] == "image/png") {
                        $image = new Zend_Pdf_Resource_Image_Png($imageFileName);
                    }
                if ($image) {
                    if ($size[1] > 200) {
                        $size[0] = $size[0] * (200 / $size[1]);
                        $size[1] = 200;
                    }
                    $this->_page->drawImage($image, $this->_pdf->getPageTop(),
                        $this->_pdf->getPageHeight() - ($size[1] / 2), $this->_pdf->getPageLeft() + ($size[0] / 2),
                        $this->_pdf->getPageHeight());
                }
            }
        }

        if ($pageNumber == 1) {
            // invoice
            $string = new XenForo_Phrase('waindigo_invoice_invoices');
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
            $fontSize = 20;
            $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
            $x = $this->_pdf->getPageWidth() - $width;
            $y = $this->_pdf->getPageHeight() - $fontSize;
            $this->_page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
            $this->_pdf->setPageTop(60);
        }

        // $totalPages = round((count($invoice['invoice_items'])-1)/11)+1;

        // invoice details
        $strings = array(
            (string) new XenForo_Phrase('waindigo_invoice_number_invoices') => $invoice['invoice_number'],
            (string) new XenForo_Phrase('date') => date("j F Y", $invoice['invoiceDate']),
            // "Page" => $pageNumber." of ".$totalPages,
            (string) new XenForo_Phrase('waindigo_page_invoices') => $pageNumber
        );

        $pageDetailsBlock = Waindigo_Pdf_Block::create('Waindigo_Invoices_Pdf_Block_Invoice_Header_PageDetails');
        $pageDetailsBlock->draw($this->_pdf, $strings);

        $this->_pdf->setPageTop($this->_pdf->getPageTop() + 20);

        if ($pageNumber > 1) {
            return;
        }

        // invoice to
        $strings = array(
            new XenForo_Phrase('waindigo_to_invoices'),
            $invoice['to_name']
        );
        if (isset($invoice['to_address'])) {
            $strings = array_merge($strings, preg_split('/$\R?^/m', $invoice['to_address']));
        }

        $addressBlock = Waindigo_Pdf_Block::create('Waindigo_Invoices_Pdf_Block_Invoice_Header_Address');
        $top1 = $addressBlock->draw($this->_pdf, $strings);

        // invoice from
        $strings = array(
            new XenForo_Phrase('waindigo_from_invoices'),
            $invoice['name']
        );
        if (isset($invoice['address'])) {
            $strings = array_merge($strings, preg_split('/$\R?^/m', $invoice['address']));
        }
        $addressBlock = Waindigo_Pdf_Block::create('Waindigo_Invoices_Pdf_Block_Invoice_Header_Address');
        $top2 = $addressBlock->draw($this->_pdf, $strings, 280);

        $this->_pdf->setPageTop(max(array(
            $top1,
            $top2
        )) + 30);
    } /* END draw */
}