<?php

class Waindigo_Invoices_Pdf_Block_Invoice_Items extends Waindigo_Pdf_Block
{

    protected $_invoice = array();

    protected $_subtotal = 0;

    protected $_vat = 0;

    public function draw(Waindigo_Pdf $pdf, array $invoice = array())
    {
        parent::draw($pdf);

        $this->_invoice = $invoice;

        $page = $this->_pdf->getCurrentPage();

        $pageWidth = $this->_pdf->getPageWidth($page);
        $pageHeight = $this->_pdf->getPageHeight($page);

        $y = $pageHeight - $this->_pdf->getPageTop();
        $x = $this->_pdf->getPageLeft();

        $itemCount = 0;
        if (isset($invoice['invoice_items'])) {
            foreach ($invoice['invoice_items'] as $invoiceItem) {
                $height = $this->_getInvoiceItemHeight($invoiceItem);

                if ($height + $this->_pdf->getPageTop() + 50 > $pageHeight) {
                    $itemCount = 0;
                    $page = $this->_pdf->addNewPage();
                }

                if ($itemCount == 0) {
                    $y = $pageHeight - $this->_pdf->getPageTop();
                    $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
                    $fontSize = 11;
                    $strings = array(
                        new XenForo_Phrase('waindigo_description_invoices') . ':',
                        new XenForo_Phrase('waindigo_item_cost_invoices') . ':',
                        new XenForo_Phrase('waindigo_quantity_invoices') . ':',
                        new XenForo_Phrase('waindigo_total_invoices') . ':'
                    );
                    $this->_drawItemColumns($page, $strings, $font, $fontSize, $x, $y);
                    $this->_pdf->setPageTop($this->_pdf->getPageTop() - 10);
                }

                $this->_drawItem($this->_pdf, $invoiceItem, $invoice);

                $itemCount++;
            }
        }

        if ($itemCount > 9) {
            $itemCount = 0;

            $page = $this->_pdf->addNewPage();
            $y = $this->getPageTop();
        }

        $strings = array();
        // if ($invoice['from_customer']['vat_number'])
        // {
        // $strings = array(
        // "Subtotal:" => number_format($subtotal, 2),
        // "VAT:" => number_format($vat, 2),
        // );
        // }
        $y = $pageHeight - $this->_pdf->getPageTop() - 30;
        $strings[new XenForo_Phrase('waindigo_total_invoices') . ':'] = number_format($this->_subtotal + $this->_vat, 2);

        foreach ($strings as $string1 => $string2) {
            $x = $pageWidth - 80;
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
            $fontSize = 11;
            $width = $this->widthForStringUsingFontSize($string1, $font, $fontSize);
            $newX = $x - $width;
            $page->setFont($font, $fontSize)->drawText($string1, $newX, $y, 'UTF-8');
            $x += 80;
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
            $fontSize = 10;
            $width = $this->widthForStringUsingFontSize($string2, $font, $fontSize);
            $newX = $x - $width;
            $page->setFont($font, $fontSize)->drawText($string2, $newX, $y, 'UTF-8');
            $y -= 20;
        }
    } /* END draw */

    protected function _getInvoiceItemHeight(array $invoiceItem)
    {
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $descriptionWidth = $this->_pdf->getPageWidth() - 270;
        $height = $this->_getWrappedTextHeight($invoiceItem['description'], $font, $fontSize, $descriptionWidth);
        $height += count($invoiceItem['invoice_item_fields']) * 12;
        return $height;
    } /* END _getInvoiceItemHeight */

    protected function _drawItem(Waindigo_Pdf $pdf, array $invoiceItem, array $invoice)
    {
        $page = $pdf->getCurrentPage();

        $x = $pdf->getPageLeft();
        $top = $pdf->getPageTop();

        $invoiceItem['cost_amount'] = str_replace(",", "", $invoiceItem['cost_amount']);
        $vatRate = 0;
        $fieldStrings = array();
        if (isset($invoiceItem['invoice_item_fields'])) {
            foreach ($invoiceItem['invoice_item_fields'] as $invoiceItemField) {
                if ($invoiceItemField['field_name'] == "VAT") {
                    $vatRate = $invoiceItemField['field_value'];
                    if ($vatRate[strlen($vatRate) - 1] == "%") {
                        $vatRate = $vatRate / 100;
                    }
                }
                $fieldStrings[] = $this->_prepareInvoiceItemField($invoiceItemField, $invoice);
            }
        }
        // if ($invoice['from_customer']['vat_number'])
        // {
        // $subtotal += $invoiceItem['cost_amount']*$invoiceItem['quantity'];
        // $vat += $invoiceItem['cost_amount']*$invoiceItem['quantity'] *
        // $vatRate;
        // }
        // else
        // {
        $this->_subtotal += $invoiceItem['cost_amount'] * $invoiceItem['quantity'] * (1 + $vatRate);
        $invoiceItem['cost_amount'] *= (1 + $vatRate);
        // }
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $top += 24;
        $strings = array(
            $invoiceItem['description'],
            $this->_currencyFormat($invoiceItem['cost_amount']),
            $invoiceItem['quantity'],
            $this->_currencyFormat($invoiceItem['cost_amount'] * $invoiceItem['quantity'])
        );
        $y = $pdf->getPageHeight($page) - $top;
        $this->_drawItemColumns($page, $strings, $font, $fontSize, $x, $y);
        $fieldStrings = array_filter($fieldStrings);
        $top = $this->_pdf->getPageTop() - 9;
        foreach ($fieldStrings as $string) {
            $top += 12;
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_ITALIC);
            $fontSize = 9;
            $y = $pdf->getPageHeight($page) - $top;
            $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
        }
        $pdf->setPageTop($top);
    } /* END _drawItem */

    protected function _currencyFormat($number)
    {
        return number_format($number, 2);
    } /* END _currencyFormat */

    protected function _drawItemColumns(Zend_Pdf_Page $page, array $strings, $font, $fontSize, $x, $y)
    {
        $pageWidth = $this->_pdf->getPageWidth($page);

        $newY = $y;
        $this->_wrapText($page, array_shift($strings), $font, $fontSize, $pageWidth - 270, $x, $newY);
        $x = $pageWidth - 160;
        foreach ($strings as $string) {
            $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
            $newX = $x - $width;
            $page->setFont($font, $fontSize)->drawText($string, $newX, $y, 'UTF-8');
            $x += 80;
        }
        $this->_pdf->setPageTop($this->_pdf->getPageHeight($page) - $newY);
    } /* END _drawItemColumns */

    protected function _prepareInvoiceItemField($invoiceItemField, $invoice)
    {
        if ($invoiceItemField['field_name'] == "VAT") {
            // if ($invoice['from_customer']['vat_number'])
            // {
            // return $invoiceItemField['field_name'] .
        // ($invoiceItemField['field_value'] ? ":
        // ".$invoiceItemField['field_value'] : "");
            // }
        } else {
            return $invoiceItemField['field_name'] .
                 ($invoiceItemField['field_value'] ? ": " . $invoiceItemField['field_value'] : "");
        }
    } /* END _prepareInvoiceItemField */

    protected function _wrapText(Zend_Pdf_Page $page, $string, $font, $fontSize, $targetWidth, $x, &$y)
    {
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $stringPieces = explode(' ', $string);
        $nextStringPieces = array();
        while ($width > $targetWidth && count($stringPieces) > 1) {
            $nextStringPieces[] = array_pop($stringPieces);
            $string = implode(' ', $stringPieces);
            $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        }
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
        $y -= $fontSize * 1.3;
        if (!empty($nextStringPieces)) {
            $nextStringPieces = array_reverse($nextStringPieces);
            $nextString = implode(' ', $nextStringPieces);
            $this->_wrapText($page, $nextString, $font, $fontSize, $targetWidth, $x, $y);
        }
    } /* END _wrapText */

    protected function _getWrappedTextHeight($string, $font, $fontSize, $targetWidth)
    {
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $stringPieces = explode(' ', $string);
        $nextStringPieces = array();
        while ($width > $targetWidth && count($stringPieces) > 1) {
            $nextStringPieces[] = array_pop($stringPieces);
            $string = implode(' ', $stringPieces);
            $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        }
        $height = $fontSize * 1.3;
        if (!empty($nextStringPieces)) {
            $nextStringPieces = array_reverse($nextStringPieces);
            $nextString = implode(' ', $nextStringPieces);
            $height += $this->_getWrappedTextHeight($nextString, $font, $fontSize, $targetWidth);
        }
        return $height;
    } /* END _getWrappedTextHeight */
}