<?php

class Waindigo_Invoices_Pdf_StandingOrder extends Waindigo_Pdf
{

    protected $_marginX = 40;

    protected $_marginY = 40;

    protected function _getPageWidth($page)
    {
        return $page->getWidth() - $this->_marginX;
    } /* END _getPageWidth */

    protected function _getPageHeight($page)
    {
        return $page->getHeight() - $this->_marginY;
    } /* END _getPageHeight */

    public function draw(array $invoice)
    {
        $page = new Zend_Pdf_Page(Zend_Pdf_Page::SIZE_A4);

        $this->_drawHeader($page, $invoice);
        $this->_drawYourBankDetails($page, $invoice);
        $this->_drawYourAccountDetails($page, $invoice);
        $this->_drawBeneficiaryAccountDetails($page, $invoice);
        $this->_drawPaymentDetails($page, $invoice);
        $this->_drawSignatures($page, $invoice);
        $this->_drawFooter($page, $invoice);

        $this->pages[] = $page;
    } /* END draw */

    protected function _drawHeader(Zend_Pdf_Page &$page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        // logo
        if (isset($invoice['from_customer']['user_id'])) {
            $imageFileName = XenForo_Model::create('XenForo_Model_Avatar')->getAvatarFilePath(
                $invoice['from_customer']['user_id'], 'l');
            if (file_exists($imageFileName)) {
                $size = getimagesize($imageFileName);
                $image = false;
                if ($size['mime'] == "image/jpeg") {
                    $image = new Zend_Pdf_Resource_Image_Jpeg($imageFileName);
                } else
                    if ($size['mime'] == "image/png") {
                        $image = new Zend_Pdf_Resource_Image_Png($imageFileName);
                    }
                if ($image) {
                    if ($size[1] > 200) {
                        $size[0] = $size[0] * (200 / $size[1]);
                        $size[1] = 200;
                    }
                    $page->drawImage($image, $this->_marginX, $pageHeight - ($size[1] / 2),
                        $this->_marginX + ($size[0] / 2), $pageHeight);
                }
            }
        }

        // standing order mandate
        $string = "Standing Order Mandate";
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 20;
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $x = $pageWidth - $width;
        $y = $pageHeight - $fontSize;
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');

        // please complete in block caps
        $string = "Please complete in BLOCK CAPITALS";
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $x = $pageWidth - $width;
        $y = $pageHeight - 60;
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
    } /* END _drawHeader */

    protected function _drawYourBankDetails(Zend_Pdf_Page $page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 14;
        $x = $this->_marginX;
        $y = $pageHeight - 130;
        $page->setFont($font, $fontSize)->drawText("Your Bank Details", $x, $y, 'UTF-8');

        $y -= 30;
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $string = "To (name of your bank):";
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $page->drawRectangle($x + $width + 5, $y + 12, $pageWidth, $y - 4, Zend_Pdf_Page::SHAPE_DRAW_STROKE);

        $y -= 20;
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $string = "Full address:";
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $page->drawRectangle($x + $width + 5, $y + 12, $pageWidth, $y - 4, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $y -= 20;
        $page->drawRectangle($x + $width + 5, $y + 12, $pageWidth, $y - 4, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
    } /* END _drawYourBankDetails */

    protected function _drawYourAccountDetails(Zend_Pdf_Page $page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 14;
        $x = $this->_marginX;
        $y = $pageHeight - 250;
        $page->setFont($font, $fontSize)->drawText("Your Account Details", $x, $y, 'UTF-8');

        $y -= 30;
        $customer = $invoice['to_customer'];
        $this->_drawAccountDetails($page, $y, $customer);
    } /* END _drawYourAccountDetails */

    protected function _drawBeneficiaryAccountDetails(Zend_Pdf_Page $page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 14;
        $x = $this->_marginX;
        $y = $pageHeight - 350;
        $page->setFont($font, $fontSize)->drawText("Beneficiary Details - Who You Want To Pay", $x, $y, 'UTF-8');

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $y -= 15;
        $page->setFont($font, $fontSize)->drawText(
            "Please set up the following standing order and debit my / our account accordingly:", $x, $y, 'UTF-8');

        $y -= 30;
        $customer = $invoice['from_customer'];
        $this->_drawAccountDetails($page, $y, $customer, $invoice['to_customer']['company_name']);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $string = "Reference:";
        $page->setFont($font, $fontSize)->drawText($string, $x, $y - 20, 'UTF-8');
    } /* END _drawBeneficiaryAccountDetails */

    protected function _drawAccountDetails(Zend_Pdf_Page $page, $y, array $customer, $reference = "")
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);
        $x = $this->_marginX;

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $string = "Account name:";
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
        $width1 = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $string = "Account number:";
        $width2 = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $page->drawRectangle($x + $width1 + 5, $y + 12, $pageWidth - $width2 - 180, $y - 4,
            Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $page->drawRectangle($x + $width1 + 5, $y - 8, $pageWidth - $width2 - 180, $y - 24,
            Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $page->setFont($font, $fontSize)->drawText(strtoupper($customer['account_name']), $x + $width1 + 10, $y, 'UTF-8');
        $page->setFont($font, $fontSize)->drawText(strtoupper($reference), $x + $width1 + 10, $y - 20, 'UTF-8');

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $x = $pageWidth - $width2 - 165;
        $page->setFont($font, $fontSize)->drawText("Sort code:", $x, $y, 'UTF-8');
        for ($i = 0; $i < 8; $i++) {
            if ($i != 2 && $i != 5) {
                $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
                $fontSize = 10;
                $page->drawRectangle($pageWidth - $i * 20, $y + 12, $pageWidth - ($i + 1) * 20 + 3, $y - 4,
                    Zend_Pdf_Page::SHAPE_DRAW_STROKE);
                if ($customer['sort_code'] != "00-00-00") {
                    $page->setFont($font, $fontSize)->drawText($customer['sort_code'][(7 - $i)], $pageWidth - $i * 20 -
                         11, $y, 'UTF-8');
                }
            } else {
                $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
                $fontSize = 11;
                $page->setFont($font, $fontSize)->drawText("-", $pageWidth - $i * 20 - 11, $y, 'UTF-8');
            }
        }

        $y -= 20;
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $page->setFont($font, $fontSize)->drawText("Account number:", $x, $y, 'UTF-8');
        for ($i = 0; $i < 8; $i++) {
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
            $fontSize = 10;
            $page->drawRectangle($pageWidth - $i * 20, $y + 12, $pageWidth - ($i + 1) * 20 + 3, $y - 4,
                Zend_Pdf_Page::SHAPE_DRAW_STROKE);
            if ($customer['account_number'] != "00000000") {
                $page->setFont($font, $fontSize)->drawText($customer['account_number'][(7 - $i)], $pageWidth - $i * 20 -
                     11, $y, 'UTF-8');
            }
        }
    } /* END _drawAccountDetails */

    protected function _drawPaymentDetails(Zend_Pdf_Page $page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $x = $this->_marginX;
        $y = $pageHeight - 470;

        $string1 = "Amount of first payment:";
        $string2 = "Amount of usual payment:";
        $string3 = "Frequency of payment:";
        $page->setFont($font, $fontSize)->drawText($string1, $x, $y, 'UTF-8');
        $page->setFont($font, $fontSize)->drawText($string2, $x, $y - 20, 'UTF-8');
        $page->setFont($font, $fontSize)->drawText($string3, $x, $y - 40, 'UTF-8');
        $width1 = max($this->widthForStringUsingFontSize($string1, $font, $fontSize),
            $this->widthForStringUsingFontSize($string2, $font, $fontSize),
            $this->widthForStringUsingFontSize($string3, $font, $fontSize));
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $page->setFont($font, $fontSize)->drawText("(Weekly, monthly, annually)", $x, $y - 55, 'UTF-8');
        $subtotal = 0;
        $vat = 0;
        if (isset($invoice['invoice_items'])) {
            foreach ($invoice['invoice_items'] as $invoiceItem) {
                $subtotal += $invoiceItem['cost_amount'] * $invoiceItem['quantity'];
                foreach ($invoiceItem['invoice_item_fields'] as $invoiceItemField) {
                    if ($invoiceItemField['field_name'] == "VAT") {
                        $vatRate = $invoiceItemField['field_value'];
                        if ($vatRate[strlen($vatRate) - 1] == "%") {
                            $vatRate = $vatRate / 100;
                        }
                        $vat += $invoiceItem['cost_amount'] * $invoiceItem['quantity'] * $vatRate;
                    }
                }
            }
        }
        $page->setFont($font, $fontSize)->drawText(number_format($subtotal + $vat, 2), $x + $width1 + 10, $y, 'UTF-8');
        $page->setFont($font, $fontSize)->drawText(number_format($subtotal + $vat, 2), $x + $width1 + 10, $y - 20, 'UTF-8');
        $page->setFont($font, $fontSize)->drawText($invoice['length_unit'] . strtoupper($invoice['length_amount']), $x + $width1 + 10, $y - 40, 'UTF-8');
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;

        $string1 = "Date of first payment:";
        $string2 = "Date of last payment:";
        $width2 = max($this->widthForStringUsingFontSize($string1, $font, $fontSize),
            $this->widthForStringUsingFontSize($string2, $font, $fontSize));
        $page->drawRectangle($x + $width1 + 5, $y + 12, $pageWidth - $width2 - 156, $y - 4,
            Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $page->drawRectangle($x + $width1 + 5, $y - 8, $pageWidth - $width2 - 156, $y - 24,
            Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $page->drawRectangle($x + $width1 + 5, $y - 28, $pageWidth - $width2 - 156, $y - 44,
            Zend_Pdf_Page::SHAPE_DRAW_STROKE);

        $x = $pageWidth - $width2 - 141;
        $page->setFont($font, $fontSize)->drawText("Date of first payment:", $x, $y, 'UTF-8');
        $invoiceDate = strftime("%d%m%y", $invoice['invoiceDate']);
        for ($i = 0; $i < 6; $i++) {
            $page->drawRectangle($pageWidth - $i * 20 - floor($i / 2) * 8, $y + 12,
                $pageWidth - ($i + 1) * 20 - floor($i / 2) * 8 + 3, $y - 4, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
            $fontSize = 10;
            $page->setFont($font, $fontSize)->drawText($invoiceDate[(5 - $i)], $pageWidth - $i * 20 - floor($i / 2) * 8 -
                 11, $y, 'UTF-8');
            if ($i == 2 || $i == 4) {
                $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
                $fontSize = 11;
                $page->setFont($font, $fontSize)->drawText("/", $pageWidth - $i * 20 - floor($i / 2) * 8 + 5, $y, 'UTF-8');
            }
        }

        $y -= 20;
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $x = $pageWidth - $width2 - 141;
        $page->setFont($font, $fontSize)->drawText("Date of last payment:", $x, $y, 'UTF-8');
        for ($i = 0; $i < 6; $i++) {
            $page->drawRectangle($pageWidth - $i * 20 - floor($i / 2) * 8, $y + 12,
                $pageWidth - ($i + 1) * 20 - floor($i / 2) * 8 + 3, $y - 4, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
            $fontSize = 10;
            $page->setFont($font, $fontSize)->drawText("X", $pageWidth - $i * 20 - floor($i / 2) * 8 - 11, $y, 'UTF-8');
            if ($i == 2 || $i == 4) {
                $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
                $fontSize = 11;
                $page->setFont($font, $fontSize)->drawText("/", $pageWidth - $i * 20 - floor($i / 2) * 8 + 5, $y, 'UTF-8');
            }
        }

        $y -= 25;
        $x = $pageWidth - $width2 - 141;
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $string = "or continue until further notice:";
        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $page->drawRectangle($x + $width + 5, $y + 12, $x + $width + 22, $y - 4, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $page->setFont($font, $fontSize)->drawText("X", $x + $width + 10, $y, 'UTF-8');
    } /* END _drawPaymentDetails */

    protected function _drawSignatures(Zend_Pdf_Page $page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 14;
        $x = $this->_marginX;
        $y = $pageHeight - 575;
        $page->setFont($font, $fontSize)->drawText("Confirmation", $x, $y, 'UTF-8');

        $y -= 30;
        $fontSize = 11;
        $string1 = "Your signature(s):";
        $string2 = "Contact telephone number:";
        $page->setFont($font, $fontSize)->drawText($string1, $x, $y, 'UTF-8');
        $page->setFont($font, $fontSize)->drawText($string2, $x, $y - 30, 'UTF-8');
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;
        $page->setFont($font, $fontSize)->drawText(
            "Where signing mandate dictates both / all to sign, all parties must authorise instruction.", $x, $y - 55, 'UTF-8');
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $width1 = max($this->widthForStringUsingFontSize($string1, $font, $fontSize),
            $this->widthForStringUsingFontSize($string2, $font, $fontSize));
        $page->drawRectangle($x + $width1 + 5, $y + 12, ($pageWidth - $width1 - $this->_marginX) / 2 + $width1 +
             $this->_marginX, $y - 14, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
        $page->drawRectangle(($pageWidth - $width1 - $this->_marginX) / 2 + $width1 + $this->_marginX + 5, $y + 12,
            $pageWidth, $y - 14, Zend_Pdf_Page::SHAPE_DRAW_STROKE);

        $string = "Date:";
        $width = $this->widthForStringUsingFontSize($string, $font, $fontSize);
        $page->drawRectangle($x + $width1 + 5, $y - 18, $pageWidth - $width - 156, $y - 34,
            Zend_Pdf_Page::SHAPE_DRAW_STROKE);

        $x = $pageWidth - $width - 141;
        $page->setFont($font, $fontSize)->drawText($string, $x, $y - 30, 'UTF-8');
        $todayDate = strftime("%d%m%y");
        for ($i = 0; $i < 6; $i++) {
            $page->drawRectangle($pageWidth - $i * 20 - floor($i / 2) * 8, $y - 18,
                $pageWidth - ($i + 1) * 20 - floor($i / 2) * 8 + 3, $y - 34, Zend_Pdf_Page::SHAPE_DRAW_STROKE);
            $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
            $fontSize = 10;
            $page->setFont($font, $fontSize)->drawText($todayDate[(5 - $i)], $pageWidth - $i * 20 - floor($i / 2) * 8 -
                 11, $y - 30, 'UTF-8');
            if ($i == 2 || $i == 4) {
                $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
                $fontSize = 11;
                $page->setFont($font, $fontSize)->drawText("/", $pageWidth - $i * 20 - floor($i / 2) * 8 + 5, $y - 30, 'UTF-8');
            }
        }
    } /* END _drawSignatures */

    protected function _drawFooter(Zend_Pdf_Page $page, array $invoice)
    {
        $pageWidth = $this->_getPageWidth($page);
        $pageHeight = $this->_getPageHeight($page);

        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $fontSize = 11;
        $x = $this->_marginX;
        $y = 70;
        $page->setFont($font, $fontSize)->drawText("Please return the completed form to:", $x, $y, 'UTF-8');

        $y -= 15;
        $font = Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA);
        $fontSize = 10;

        // invoice from
        $strings[] = $invoice['from_customer']['company_name'];
        if ($invoice['from_customer']['address_line1'])
            $strings[] = $invoice['from_customer']['address_line1'];
        if ($invoice['from_customer']['address_line2'])
            $strings[] = $invoice['from_customer']['address_line2'];
        if ($invoice['from_customer']['city'])
            $strings[] = $invoice['from_customer']['city'];
        if ($invoice['from_customer']['region'])
            $strings[] = $invoice['from_customer']['region'];
        if ($invoice['from_customer']['postal_code'])
            $strings[] = $invoice['from_customer']['postal_code'];
        $string = implode(", ", $strings) . ".";

        $page->setFont($font, $fontSize)->drawText($string, $x, $y, 'UTF-8');
    } /* END _drawFooter */
}