<?php
abstract class Waindigo_Pdf extends Zend_Pdf
{
	/**
	 * Standard approach to caching other model objects for the lifetime of the model.
	 *
	 * @var array
	 */
	protected $_modelCache = array();

	protected $_pageNumber = 0;

	protected $_marginTop = 40;
	protected $_marginLeft = 40;
	protected $_marginRight = 40;
	protected $_marginBottom = 40;

	protected $_top = 0;
	protected $_left = 0;

	/**
	 * Gets the specified model object from the cache. If it does not exist,
	 * it will be instantiated.
	 *
	 * @param string $class Name of the class to load
	 *
	 * @return XenForo_Model
	 */
	public function getModelFromCache($class)
	{
		if (!isset($this->_modelCache[$class])) {
			$this->_modelCache[$class] = XenForo_Model::create($class);
		}

		return $this->_modelCache[$class];
	} /* END getModelFromCache */

	/**
	 * Factory method to get the named model. The class must exist or be autoloadable
	 * or an exception will be thrown.
	 *
	 * @param string Class to load
	 *
	 * @return Waindigo_Pdf
	 */
	public static function create($class)
	{
		$createClass = XenForo_Application::resolveDynamicClass($class, 'pdf_waindigo');
		if (!$createClass) {
			throw new XenForo_Exception("Invalid model '$class' specified");
		}

		return new $createClass;
	} /* END create */

	public function addNewPage()
	{
	    $page = $this->newPage(Zend_Pdf_Page::SIZE_A4);
	    $this->pages[] = $page;
	    $this->_pageNumber++;
	    $this->_top = $this->_marginTop;
	    $this->_left = $this->_marginLeft;
	    $this->_drawHeader();
	    return $page;
	} /* END addNewPage */

	abstract public function draw();

	abstract protected function _drawHeader();

	abstract protected function _drawFooter();

	public function setPageTop($top)
	{
	    $this->_top = $top;
	} /* END setPageTop */

	public function getPageTop()
	{
	    return $this->_top;
	} /* END getPageTop */

	public function setPageLeft($left)
	{
	    $this->_left = $left;
	} /* END setPageLeft */

	public function getPageLeft()
	{
	    return $this->_left;
	} /* END getPageLeft */

	public function getCurrentPage()
	{
	    if (empty($this->pages)) {
	        $page = $this->addNewPage();
	    } else {
	        $page = end($this->pages);
	    }
	    return $page;
	} /* END getCurrentPage */

	public function getPageWidth(Zend_Pdf_Page $page = null, $margin = true)
	{
	    if (!$page) {
	        $page = $this->getCurrentPage();
	    }
	    return $page->getWidth() - ($margin ? $this->_marginRight : 0);
	} /* END getPageWidth */

	public function getPageHeight(Zend_Pdf_Page $page = null, $margin = true)
	{
		if (!$page) {
	        $page = $this->getCurrentPage();
	    }
	    return $page->getHeight() - ($margin ? $this->_marginBottom : 0);
	} /* END getPageHeight */
}